@@ -13,6 +13,15 @@
 // limitations under the License.
 
 //! OpenStack Identity V3 API support for access tokens.
+//!
+//! Start with creating an [Identity](struct.Identity.html) object which will
+//! guide you through setting all necessary values.
+//! [IdentityAuthMethod](struct.IdentityAuthMethod.html) is the actual
+//! implementation of the authentication [method](../trait.Method.html) trait.
+//!
+//! Note that as of now, only project-scoped tokens are supported.
+//! An attempt to create unscoped tokens always fails. This restriction may
+//! be lifted in the future.
 
 use std::env;
 use std::io::Read;
@@ -24,10 +33,11 @@ use hyper::error::ParseError;
 use hyper::header::ContentType;
 use hyper::status::StatusCode;
 
-use super::super::{ApiError, Session};
-use super::super::identity::catalog;
-use super::super::identity::protocol;
-use super::base::{AuthMethod, SimpleAuthToken, SubjectTokenHeader};
+use super::super::{ApiError, ApiResult, Session};
+use super::super::identity::{catalog, protocol};
+use super::{Method, SimpleToken};
+
+use ApiError::InsufficientCredentials;
 
 
 const MISSING_USER: &'static str = "User information required";
@@ -44,9 +54,9 @@ pub struct Identity {
     project_scope: Option<protocol::ProjectScope>
 }
 
-/// Authentication method using Identity API V3.
+/// Authentication method implementation using Identity API V3.
 ///
-/// Should be created via Identity struct methods.
+/// Has to be created via [Identity object](struct.Identity.html) methods.
 #[derive(Clone, Debug)]
 pub struct IdentityAuthMethod {
     auth_url: Url,
@@ -93,19 +103,23 @@ impl Identity {
     }
 
     /// Create an authentication method based on provided information.
-    pub fn create(self) -> Result<IdentityAuthMethod, ApiError> {
+    pub fn create(self) -> ApiResult<IdentityAuthMethod> {
         /// TODO: support more authentication methods (at least a token)
         let password_identity = match self.password_identity {
             Some(p) => p,
             None =>
-                return Err(ApiError::InsufficientCredentials(MISSING_USER))
+                return Err(
+                    InsufficientCredentials(String::from(MISSING_USER))
+                )
         };
 
         /// TODO: support unscoped tokens
         let project_scope = match self.project_scope {
             Some(p) => p,
             None =>
-                return Err(ApiError::InsufficientCredentials(MISSING_SCOPE))
+                return Err(
+                    InsufficientCredentials(String::from(MISSING_SCOPE))
+                )
         };
 
         Ok(IdentityAuthMethod::new(self.auth_url, password_identity,
@@ -113,7 +127,7 @@ impl Identity {
     }
 
     /// Create an authentication method from environment variables.
-    pub fn from_env() -> Result<IdentityAuthMethod, ApiError> {
+    pub fn from_env() -> ApiResult<IdentityAuthMethod> {
         let auth_url = try!(_get_env("OS_AUTH_URL"));
         let id = match Identity::new(&auth_url) {
             Ok(x) => x,
@@ -136,9 +150,10 @@ impl Identity {
     }
 }
 
-fn _get_env(name: &str) -> Result<String, ApiError> {
-    env::var(name).or(
-        Err(ApiError::InsufficientCredentials(MISSING_ENV_VARS)))
+fn _get_env(name: &str) -> ApiResult<String> {
+    env::var(name).or(Err(
+            InsufficientCredentials(String::from(MISSING_ENV_VARS))
+    ))
 }
 
 impl IdentityAuthMethod {
@@ -162,13 +177,14 @@ impl IdentityAuthMethod {
     }
 
     fn token_from_response(&self, mut resp: Response)
-            -> Result<SimpleAuthToken, ApiError> {
+            -> ApiResult<SimpleToken> {
         let mut resp_body = String::new();
         let _ignored = try!(resp.read_to_string(&mut resp_body));
 
         let token_value = match resp.status {
             StatusCode::Ok | StatusCode::Created => {
-                let header: Option<&SubjectTokenHeader> = resp.headers.get();
+                let header: Option<&protocol::SubjectTokenHeader> =
+                    resp.headers.get();
                 match header {
                     Some(ref value) => value.0.clone(),
                     None => return Err(
@@ -178,12 +194,12 @@ impl IdentityAuthMethod {
             StatusCode::Unauthorized => {
                 warn!("Invalid credentials for user {}",
                       self.body.auth.identity.password.user.name);
-                return Err(ApiError::Unauthorized);
+                return Err(ApiError::HttpError(resp.status, resp));
             },
             other => {
                 error!("Unexpected HTTP error {} when getting a token for {}",
                        other, self.body.auth.identity.password.user.name);
-                return Err(ApiError::HttpError(resp));
+                return Err(ApiError::HttpError(resp.status, resp));
             }
         };
 
@@ -193,16 +209,16 @@ impl IdentityAuthMethod {
 
         // TODO: detect expiration time
         // TODO: do something useful about the body
-        Ok(SimpleAuthToken(token_value))
+        Ok(SimpleToken(token_value))
     }
 }
 
-impl AuthMethod for IdentityAuthMethod {
+impl Method for IdentityAuthMethod {
     // TODO: implement its own token type
-    type TokenType = SimpleAuthToken;
+    type TokenType = SimpleToken;
 
     /// Verify authentication and generate an auth token.
-    fn get_token(&self, client: &Client) -> Result<SimpleAuthToken, ApiError> {
+    fn get_token(&self, client: &Client) -> ApiResult<SimpleToken> {
         debug!("Requesting a token for user {} from {}",
                self.body.auth.identity.password.user.name,
                self.token_endpoint);
@@ -219,7 +235,7 @@ impl AuthMethod for IdentityAuthMethod {
                     endpoint_interface: Option<&str>,
                     region: Option<&str>,
                     client: &Session<IdentityAuthMethod>)
-            -> Result<Url, ApiError> {
+            -> ApiResult<Url> {
         let real_interface = endpoint_interface.unwrap_or("public");
         let cat = try!(catalog::get_service_catalog(&self.auth_url, client));
         let endp = try!(catalog::find_endpoint(&cat, service_type,
@@ -234,9 +250,10 @@ pub mod test {
     #![allow(unused_results)]
 
     use hyper;
+    use hyper::status::StatusCode;
 
     use super::super::super::{ApiError, Session};
-    use super::super::base::{AuthMethod, AuthToken, SimpleAuthToken};
+    use super::super::{Method, Token, SimpleToken};
     use super::Identity;
 
     mock_connector!(MockToken {
@@ -368,7 +385,7 @@ pub mod test {
             .create().unwrap();
         let cli = hyper::Client::with_connector(MockToken::default());
         match id.get_token(&cli).err().unwrap() {
-            ApiError::Unauthorized => (),
+            ApiError::HttpError(StatusCode::Unauthorized, ..) => (),
             other => panic!("Unexpected {}", other)
         };
     }
@@ -381,9 +398,7 @@ pub mod test {
             .create().unwrap();
         let cli = hyper::Client::with_connector(MockToken::default());
         match id.get_token(&cli).err().unwrap() {
-            ApiError::HttpError(ref resp) => {
-                assert_eq!(resp.status, hyper::NotFound);
-            },
+            ApiError::HttpError(hyper::NotFound, ..) => (),
             other => panic!("Unexpected {}", other)
         };
     }
@@ -395,7 +410,7 @@ pub mod test {
             .with_project_scope("cool project", "example.com")
             .create().unwrap();
         let cli = hyper::Client::with_connector(MockCatalog::default());
-        let token = SimpleAuthToken(String::from("abcdef"));
+        let token = SimpleToken(String::from("abcdef"));
         let session = Session::new_with_params(id, cli, token);
 
         let e1 = session.get_endpoint("identity", None, None).unwrap();
@@ -415,7 +430,7 @@ pub mod test {
             .with_project_scope("cool project", "example.com")
             .create().unwrap();
         let cli = hyper::Client::with_connector(MockCatalog::default());
-        let token = SimpleAuthToken(String::from("abcdef"));
+        let token = SimpleToken(String::from("abcdef"));
         let session = Session::new_with_params(id, cli, token);
 
         match session.get_endpoint("foo", None, None).err().unwrap() {
