@@ -21,37 +21,62 @@ use std::io;
 use hyper::Error as HttpClientError;
 use hyper::client::Response;
 use hyper::error::ParseError;
+use hyper::status::StatusCode;
 use serde_json::Error as JsonError;
 
 
+/// Trait representing a service type.
+pub trait ServiceType {
+    /// Service type to pass to the catalog.
+    fn catalog_type() -> &'static str;
+
+    /// Version suffix to append to the endpoint.
+    fn version_suffix() -> Option<&'static str>;
+}
+
+
 /// Error from an OpenStack API call.
 #[derive(Debug)]
 pub enum ApiError {
     /// Insufficient credentials passed to make authentication request.
-    InsufficientCredentials(&'static str),
+    ///
+    /// Contains the error message.
+    InsufficientCredentials(String),
+
     /// Requested service endpoint was not found.
+    ///
+    /// Contains the failed endpoint name.
     EndpointNotFound(String),
-    /// Authentication rejected (invalid credentials or token).
-    Unauthorized,
-    /// Generic HTTP error (not covered by EndpointNotFound and Unauthorized).
-    HttpError(Response),
+
+    /// Invalid value passed to one of paremeters.
+    ///
+    /// Contains the error message.
+    InvalidParameterValue(String),
+
+    /// Generic HTTP error.
+    HttpError(StatusCode, Response),
+
     /// Protocol-level error reported by underlying HTTP library.
     ProtocolError(HttpClientError),
+
     /// JSON parsing failed.
     InvalidJson(JsonError)
 }
 
+/// Result of an API call.
+pub type ApiResult<T> = Result<T, ApiError>;
+
 impl fmt::Display for ApiError {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
-            ApiError::InsufficientCredentials(msg) =>
+            ApiError::InsufficientCredentials(ref msg) =>
                 write!(f, "Insufficient credentials provided: {}", msg),
             ApiError::EndpointNotFound(ref endp) =>
                 write!(f, "Requested endpoint {} was not found", endp),
-            ApiError::Unauthorized =>
-                write!(f, "Authentication failed"),
-            ApiError::HttpError(ref resp) =>
-                write!(f, "HTTP error {}", resp.status),
+            ApiError::InvalidParameterValue(ref msg) =>
+                write!(f, "Passed parameters are invalid: {}", msg),
+            ApiError::HttpError(status, ..) =>
+                write!(f, "HTTP error {}", status),
             ApiError::ProtocolError(ref e) => fmt::Display::fmt(e, f),
             ApiError::InvalidJson(ref e) => fmt::Display::fmt(e, f)
         }
@@ -63,8 +88,10 @@ impl Error for ApiError {
         match *self {
             ApiError::InsufficientCredentials(..) =>
                 "Insufficient credentials provided",
-            ApiError::EndpointNotFound(..) => "Requested endpoint was not found",
-            ApiError::Unauthorized => "Authentication failed",
+            ApiError::EndpointNotFound(..) =>
+                "Requested endpoint was not found",
+            ApiError::InvalidParameterValue(..) =>
+                "Invalid values passed for parameters",
             ApiError::HttpError(..) => "HTTP error",
             ApiError::ProtocolError(ref e) => e.description(),
             ApiError::InvalidJson(ref e) => e.description()
