@@ -16,6 +16,11 @@
 //!
 //! The goal of this project is to provide a simple API for working with
 //! OpenStack clouds.
+//!
+//! # Features
+//!
+//! * [Authentication](auth/index.html)
+//! * Basic [compute API support](compute/index.html)
 
 #![crate_name = "openstack"]
 #![crate_type = "lib"]
@@ -68,6 +73,7 @@ extern crate serde;
 extern crate serde_derive;
 extern crate serde_json;
 extern crate time;
+extern crate uuid;
 
 #[cfg(test)] #[macro_use]
 extern crate yup_hyper_mock;
@@ -76,9 +82,11 @@ pub mod auth;
 mod common;
 #[cfg(feature = "compute")]
 pub mod compute;
-pub mod identity;
+mod identity;
 pub mod session;
 mod utils;
 
 pub use common::ApiError;
-pub use session::{ApiVersion, Session};
+pub use common::ApiResult;
+pub use common::ServiceType;
+pub use session::Session;
