@@ -17,7 +17,6 @@
 //! The Session object serves as a wrapper around an HTTP(s) client, handling
 //! authentication, accessing the service catalog and token refresh.
 
-use std::cell::RefCell;
 use std::fmt;
 use std::marker::PhantomData;
 
@@ -28,8 +27,9 @@ use hyper::method::Method;
 use serde::Deserialize;
 use serde_json;
 
-use super::ApiError;
-use super::auth::base::{AuthMethod, AuthTokenHeader};
+use super::{ApiError, ApiResult, ServiceType};
+use super::auth::Method as AuthMethod;
+use super::identity::protocol;
 use super::utils;
 
 
@@ -46,33 +46,33 @@ pub struct AuthenticatedRequestBuilder<'a, A: AuthMethod + 'a> {
 ///
 /// Owns a token and an underlying client.
 #[derive(Debug)]
-pub struct Session<A: AuthMethod> {
-    auth_method: A,
+pub struct Session<Auth: AuthMethod> {
+    auth_method: Auth,
     client: Client,
-    cached_token: RefCell<Option<A::TokenType>>
+    cached_token: utils::ValueCache<Auth::TokenType>
 }
 
-impl<'a, A: AuthMethod> AuthenticatedRequestBuilder<'a, A> {
+impl<'a, Auth: AuthMethod> AuthenticatedRequestBuilder<'a, Auth> {
     /// Send this request.
-    pub fn send(self) -> Result<Response, ApiError> {
+    pub fn send(self) -> ApiResult<Response> {
         let resp = try!(self.send_unchecked());
         if resp.status.is_success() {
             Ok(resp)
         } else {
-            Err(ApiError::HttpError(resp))
+            Err(ApiError::HttpError(resp.status, resp))
         }
     }
 
     /// Send this request without checking on status code.
-    pub fn send_unchecked(self) -> Result<Response, ApiError> {
+    pub fn send_unchecked(self) -> ApiResult<Response> {
         let token = try!(self.parent.auth_token());
-        let hdr = AuthTokenHeader(token.into());
+        let hdr = protocol::AuthTokenHeader(token.into());
         self.inner.header(hdr).send().map_err(From::from)
     }
 
     /// Add body to the request.
     pub fn body<B: Into<Body<'a>>>(self, body: B)
-            -> AuthenticatedRequestBuilder<'a, A> {
+            -> AuthenticatedRequestBuilder<'a, Auth> {
         AuthenticatedRequestBuilder {
             inner: self.inner.body(body),
             .. self
@@ -81,7 +81,7 @@ impl<'a, A: AuthMethod> AuthenticatedRequestBuilder<'a, A> {
 
     /// Add additional headers to the request.
     pub fn headers(self, headers: Headers)
-            -> AuthenticatedRequestBuilder<'a, A> {
+            -> AuthenticatedRequestBuilder<'a, Auth> {
         AuthenticatedRequestBuilder {
             inner: self.inner.headers(headers),
             .. self
@@ -92,7 +92,7 @@ impl<'a, A: AuthMethod> AuthenticatedRequestBuilder<'a, A> {
     ///
     /// Note that X-Auth-Token is always overwritten with a token in use.
     pub fn header<H: Header + HeaderFormat>(self, header: H)
-            -> AuthenticatedRequestBuilder<'a, A> {
+            -> AuthenticatedRequestBuilder<'a, Auth> {
         AuthenticatedRequestBuilder {
             inner: self.inner.header(header),
             .. self
@@ -101,33 +101,33 @@ impl<'a, A: AuthMethod> AuthenticatedRequestBuilder<'a, A> {
 }
 
 
-impl<'a, A: AuthMethod + 'a> Session<A> {
+impl<'a, Auth: AuthMethod + 'a> Session<Auth> {
     /// Create a new session with a given authentication plugin.
-    pub fn new(auth_method: A) -> Session<A> {
+    pub fn new(auth_method: Auth) -> Session<Auth> {
         Session {
             auth_method: auth_method,
             client: utils::http_client(),
-            cached_token: RefCell::new(None)
+            cached_token: utils::ValueCache::new(None)
         }
     }
 
     /// Get a clone of the authentication token.
-    pub fn auth_token(&self) -> Result<A::TokenType, ApiError> {
+    pub fn auth_token(&self) -> ApiResult<Auth::TokenType> {
         try!(self.refresh_token());
-        Ok(self.cached_token.borrow().clone().unwrap())
+        Ok(self.cached_token.get().unwrap())
     }
 
     /// Get an endpoint URL.
     pub fn get_endpoint(&self, service_type: &str,
                         endpoint_interface: Option<&str>,
-                        region: Option<&str>) -> Result<Url, ApiError> {
+                        region: Option<&str>) -> ApiResult<Url> {
         self.auth_method.get_endpoint(service_type, endpoint_interface,
                                       region, &self)
     }
 
     /// A wrapper for HTTP request.
     pub fn request<U: IntoUrl>(&'a self, method: Method, url: U)
-            -> AuthenticatedRequestBuilder<'a, A> {
+            -> AuthenticatedRequestBuilder<'a, Auth> {
         AuthenticatedRequestBuilder {
             parent: self,
             inner: self.client.request(method, url)
@@ -137,26 +137,19 @@ impl<'a, A: AuthMethod + 'a> Session<A> {
     // Private and test-only
 
     #[cfg(test)]
-    pub fn new_with_params(auth_method: A, client: Client,
-                           token: A::TokenType) -> Session<A> {
+    pub fn new_with_params(auth_method: Auth, client: Client,
+                           token: Auth::TokenType) -> Session<Auth> {
         Session {
             auth_method: auth_method,
             client: client,
-            cached_token: RefCell::new(Some(token))
+            cached_token: utils::ValueCache::new(Some(token))
         }
     }
 
-    fn refresh_token(&self) -> Result<(), ApiError> {
-        let mut cached_token = self.cached_token.borrow_mut();
-        if cached_token.is_some() {
-            return Ok(())
-        }
-
-        // TODO: check expires_at
-
-        let new_token = try!(self.auth_method.get_token(&self.client));
-        *cached_token = Some(new_token);
-        Ok(())
+    fn refresh_token(&self) -> ApiResult<()> {
+        self.cached_token.ensure_value(|| {
+            self.auth_method.get_token(&self.client)
+        })
     }
 }
 
@@ -174,27 +167,18 @@ impl fmt::Display for ApiVersion {
     }
 }
 
-/// Trait representing a service type.
-pub trait ServiceType {
-    /// Service type to pass to the catalog.
-    fn catalog_type() -> &'static str;
-
-    /// Version suffix to append to the endpoint.
-    fn version_suffix() -> Option<&'static str>;
-}
-
 /// Low-level API calls.
 #[derive(Debug)]
-pub struct ServiceApi<'a, A: AuthMethod + 'a, S> {
-    session: &'a Session<A>,
+pub struct ServiceApi<'a, Auth: AuthMethod + 'a, S> {
+    session: &'a Session<Auth>,
     service_type: PhantomData<S>,
     endpoint_interface: Option<String>,
     region: Option<String>
 }
 
-impl<'a, A: AuthMethod + 'a, S: ServiceType> ServiceApi<'a, A, S> {
+impl<'a, Auth: AuthMethod + 'a, S: ServiceType> ServiceApi<'a, Auth, S> {
     /// Create a new API instance using the given session.
-    pub fn new(session: &'a Session<A>) -> ServiceApi<'a, A, S> {
+    pub fn new(session: &'a Session<Auth>) -> ServiceApi<'a, Auth, S> {
         ServiceApi::new_with_endpoint_params(session, None, None)
     }
 
@@ -202,10 +186,10 @@ impl<'a, A: AuthMethod + 'a, S: ServiceType> ServiceApi<'a, A, S> {
     ///
     /// This variant allows passing an endpoint type (defaults to public),
     /// and region (defaults to any).
-    pub fn new_with_endpoint_params(session: &'a Session<A>,
+    pub fn new_with_endpoint_params(session: &'a Session<Auth>,
                                     endpoint_interface: Option<&str>,
                                     region: Option<&str>)
-            -> ServiceApi<'a, A, S> {
+            -> ServiceApi<'a, Auth, S> {
         ServiceApi {
             session: session,
             service_type: PhantomData,
@@ -215,7 +199,7 @@ impl<'a, A: AuthMethod + 'a, S: ServiceType> ServiceApi<'a, A, S> {
     }
 
     /// Get an endpoint with version suffix and given path appended.
-    pub fn get_endpoint(&self, path: &str) -> Result<Url, ApiError> {
+    pub fn get_endpoint(&self, path: &str) -> ApiResult<Url> {
         let endpoint = try!(self.session.get_endpoint(
                 S::catalog_type(),
                 self.endpoint_interface.as_ref().map(String::as_str),
@@ -235,13 +219,31 @@ impl<'a, A: AuthMethod + 'a, S: ServiceType> ServiceApi<'a, A, S> {
     }
 
     /// List entities.
-    pub fn list<R: Deserialize>(&self, path: &str) -> Result<R, ApiError> {
+    pub fn list<R: Deserialize>(&self, path: &str) -> ApiResult<R> {
+        // TODO: filtering
         let url = try!(self.get_endpoint(path));
         debug!("Listing entities from {}", url);
         let resp = try!(self.session.request(Get, url).send());
         let root = try!(serde_json::from_reader(resp));
         Ok(root)
     }
+
+    /// Get one entity.
+    pub fn get<R: Deserialize, Id: utils::IntoId>(&self, path: &str, id: Id)
+            -> ApiResult<R> {
+        // Url expects trailing /
+        let root_path = if path.ends_with("/") {
+            String::from(path)
+        } else {
+            format!("{}/", path)
+        };
+        let url = try!(self.get_endpoint(&root_path));
+        let url_with_id = try!(url.join(&id.into_id()));
+        debug!("Get one entity from {}", url);
+        let resp = try!(self.session.request(Get, url_with_id).send());
+        let root = try!(serde_json::from_reader(resp));
+        Ok(root)
+    }
 }
 
 #[cfg(test)]
@@ -249,18 +251,18 @@ pub mod test {
     #![allow(missing_debug_implementations)]
     #![allow(unused_results)]
 
-    use std::cell::RefCell;
     use std::io::Read;
 
     use hyper;
     use hyper::header::ContentLength;
+    use hyper::status::StatusCode;
 
-    use super::Session;
-    use super::super::auth::base::{AuthToken, NoAuth, SimpleAuthToken};
+    use super::{ApiError, Session};
+    use super::super::auth::{Token, NoAuth, SimpleToken};
     use super::super::utils;
 
     pub fn new_session(token: &str) -> Session<NoAuth> {
-        let token = SimpleAuthToken(String::from(token));
+        let token = SimpleToken(String::from(token));
         Session::new_with_params(NoAuth::new("http://127.0.0.1/").unwrap(),
                                  utils::http_client(), token)
     }
@@ -270,6 +272,10 @@ pub mod test {
                                Server: Mock.Mock\r\n\
                                \r\n\
                                {}"
+        "http://127.0.0.2" => "HTTP/1.1 404 NOT FOUND\r\n\
+                               Server: Mock.Mock\r\n\
+                               \r\n\
+                               {}"
     });
 
     #[test]
@@ -293,7 +299,7 @@ pub mod test {
         let s = Session {
             auth_method: NoAuth::new("http://127.0.0.1/").unwrap(),
             client: cli,
-            cached_token: RefCell::new(None)
+            cached_token: utils::ValueCache::new(None)
         };
 
         let mut resp = s.request(hyper::Post, "http://127.0.0.1/")
@@ -303,4 +309,42 @@ pub mod test {
         resp.read_to_string(&mut s).unwrap();
         assert_eq!(&s, "{}");
     }
+
+    #[test]
+    fn test_session_request_error() {
+        let cli = hyper::Client::with_connector(MockHttp::default());
+        let s = Session {
+            auth_method: NoAuth::new("http://127.0.0.2/").unwrap(),
+            client: cli,
+            cached_token: utils::ValueCache::new(None)
+        };
+
+        let err = s.request(hyper::Post, "http://127.0.0.2/")
+            .body("body").header(ContentLength(4u64)).send().err().unwrap();
+
+        match err {
+            ApiError::HttpError(StatusCode::NotFound, ..) => (),
+            other => panic!("Unexpected {}", other)
+        }
+    }
+
+    #[test]
+    fn test_session_request_unchecked_error() {
+        let cli = hyper::Client::with_connector(MockHttp::default());
+        let s = Session {
+            auth_method: NoAuth::new("http://127.0.0.2/").unwrap(),
+            client: cli,
+            cached_token: utils::ValueCache::new(None)
+        };
+
+        let mut resp = s.request(hyper::Post, "http://127.0.0.2/")
+            .body("body").header(ContentLength(4u64)).send_unchecked()
+            .unwrap();
+
+        assert_eq!(resp.status, StatusCode::NotFound);
+
+        let mut s = String::new();
+        resp.read_to_string(&mut s).unwrap();
+        assert_eq!(&s, "{}");
+    }
 }
